REQUIREMENTS_LIST = [
    'django',
    'pre-commit',
]


class requirements:
    '''
        Object used to handle the requirements list
    '''
    requirements_list = REQUIREMENTS_LIST

    @classmethod
    def get_requirements_as_string(cls):
        '''
            Return a string fitting the requirements.txt
        '''
        requirements = '# Run pip install -r requirements.txt\n'
        for requirement in cls.requirements_list:
            requirements += f'{requirement}\n'
        return requirements



# Built-in 
import re
import subprocess

# Add parent path
import sys
sys.path.insert(0, re.findall('(^.+)/', sys.path[0])[0])

# Own modules
from conf import requirements
from cprint import cprint


# Upgrade pip
cprint.info('\nUpgrading pip... ')
command = 'python -m pip install --upgrade pip'
subprocess.run(command.split())
cprint.success('[Success]')

# Install requirements
for requirement in requirements.requirements_list:
    cprint.info(f'\nInstalling {requirement}... ')
    command = f'pip install {requirement}'
    effort = subprocess.run(command.split())
    if effort.returncode == 0:
        cprint.success('[Success]')
    else:
        cprint.error('[Error]')

cprint.info(f'\nAll modules installed ')
subprocess.run('pip list', shell=True)

# Start django project
project_name = sys.argv[1]
cprint.info(f'\nCreate the django project "{project_name}"... ')
command = f'django-admin startproject {project_name}'
subprocess.run(command.split())
cprint.success('[Success]')

# Run migrations
cprint.info(f'\nRunning migrations... ')
command = f'cd {project_name} && python3 manage.py migrate'
subprocess.run(command, shell=True)
cprint.success('[Success]')

# Start app Home
cprint.info(f'\nStarting app "home"... ')
command = f'cd {project_name} && python3 manage.py startapp home'
subprocess.run(command, shell=True)
cprint.success('[Success]')

# Add .editorconfig file
cprint.info(f'\nAdding .editorconfig to the project... ')
command = f'cp {sys.path[0]}/files/.editorconfig {project_name}/'
subprocess.run(command, shell=True)
cprint.success('[Success]')

# Set up app Home
cprint.info(f'\nSetting up app "home"... ')

# Templates
command = f'mkdir {project_name}/home/templates/ &&\
            mkdir {project_name}/home/templates/home/ &&\
            cp {sys.path[0]}/files/home/templates/home/main.html {project_name}/home/templates/home/'
subprocess.run(command, shell=True)

# Static
command = f'mkdir {project_name}/home/static/ &&\
            mkdir {project_name}/home/static/home/ &&\
            cp {sys.path[0]}/files/home/static/favicon.ico {project_name}/home/static/'
subprocess.run(command, shell=True)

# replace views.py, urls.py and context_processors.py
command = f'cp {sys.path[0]}/files/home/*.py {project_name}/home/'
subprocess.run(command, shell=True)
cprint.success('[Success]')

# Set up Core
cprint.info(f'\nSetting up core... ')
# replace urls.py
command = f'cp {sys.path[0]}/files/core/*.py {project_name}/{project_name}/'
subprocess.run(command, shell=True)

# Edit settings.py
new_lines = [
    (31, ""),
    (32, "APP_NAME = 'Project Name'"),
    (41, ""),
    (42, "    # Own Apps"),
    (43, "    'home',"),
    (69, "                'home.context_processors.settings',"),
]

# Open file, make a list and close it
with open(f'{project_name}/{project_name}/settings.py', 'r') as settings_file:
    settings_list = settings_file.read().split('\n')
    settings_file.close()

# Add new lines
for i, line in new_lines:
    settings_list.insert(i, line)

# Convert to string
setting_string = ''
for line in settings_list:
    setting_string += (line + '\n')

# Rewrite settings.py
with open(f'{project_name}/{project_name}/settings.py', 'w') as settings_file:
    settings_file.write(setting_string[:-1])

cprint.success('[Success]')



# coding: utf8

# Adaptation from:
# https://github.com/evasseure/cprint/blob/master/cprint/cprint.py

from __future__ import print_function, unicode_literals
import sys


class cprint(object):

    colors = {
        'NONE': '\033[0m',
        'SUCCESS': '\033[32m',
        'ERROR': '\033[91m',
        'INFO': '\033[34m',
        'ENDC': '\033[0m'
    }

    def __init__(self, str):
        """
            Prints in white to stdout
        """
        print(str, file=sys.stdout)
        del self

    @classmethod
    def _get_repr(cls, arg):
        if isinstance(arg, str):
            return arg
        return repr(arg)

    @classmethod
    def success(cls, arg, *args, **kwargs):
        """
            Prints in green to stdout
        """
        print(
            cprint.colors['SUCCESS'] +
            cls._get_repr(arg) +
            cprint.colors['ENDC'],
            file=sys.stdout,
            *
            args,
            **kwargs)

    @classmethod
    def error(cls, arg, interrupt=False, 
              fatal_message="Fatal error: Program stopped.", *args, **kwargs):
        """
            Prints in brown to stderr
            interrupt=True: stops the program
        """
        print(
            cprint.colors['ERROR'] +
            cls._get_repr(arg) +
            cprint.colors['ENDC'],
            file=sys.stderr,
            *
            args,
            **kwargs)
        if interrupt:
            print(cprint.colors['ERROR'] + fatal_message +
                  cprint.colors['ENDC'],
                  file=sys.stderr)

    @classmethod
    def info(cls, arg, *args, **kwargs):
        """
            Prints in blue to stdout
        """
        print(
            cprint.colors['INFO'] +
            cls._get_repr(arg) +
            cprint.colors['ENDC'],
            file=sys.stdout,
            *
            args,
            **kwargs)



import re


def project_to_venv(project_name):
    '''Takes a string like "project_name" and convert to "ProjectName", or
    "projectname" to "Projectname"
    '''
    venv_name = ''
    for word in re.findall('[a-zA-Z]+', project_name):
        venv_name += word.title()
    return venv_name



# Built-in modules
import sys
import requests

# Own modules
from .cprint import cprint

# Check if there internet connection
# https://www.kite.com/python/answers/how-to-check-internet-connection-in-python
url = "http://www.google.com"
timeout = 5
try:
    request = requests.get(url, timeout=timeout)
    cprint.success(''.center(30, '-'))
    cprint.success('|' + 'Connected to the Internet'.center(28, ' ') + '|')
    cprint.success(''.center(30, '-'))
except (requests.ConnectionError, requests.Timeout) as exception:
    cprint.error(''.center(30, '-'))
    cprint.error('|' + 'No internet connection'.center(28, ' ') + '|')
    cprint.error(''.center(30, '-'))
    exit(1)

if __name__ == '__main__':
    from .core import main
    main(sys.argv)


from django.shortcuts import render
from django.views import View
from django.conf import settings

class HomeView(View):

    template = 'home/main.html'

    def get(self, request):
        host = request.get_host()
        islocal = host.find('localhost') >= 0 or host.find('127.0.0.1') >= 0
        context = {
            'installed': settings.INSTALLED_APPS,
            'islocal': islocal
        }
        return render(request, self.template, context)



import os
import subprocess

from .cprint import cprint

from .conf import requirements


def run(project_name, venv_name):
    '''Create a virtual envioremnt and a django project'''

    # Create requirements.txt
    cprint.info('\nCreate "requirements.txt"... ')
    with open(f'requirements.txt', 'w') as requirements_file:
        requirements_file.write(requirements.get_requirements_as_string())
        requirements_file.close()
    cprint.success('[Success]')

    # Create virtual enviorment
    venv_name = f'venv{venv_name}'
    cprint.info(f'\nCreate the virtual enviorment "{venv_name}"... ')
    command = f'python3 -m venv {venv_name}'
    start = subprocess.run(command.split())
    if start.returncode == 0:
        cprint.success('[Success]')

    # Install modules from conf.REQUIREMENTS 
    folder = '/'.join(__file__.split('/')[:-1])
    command = f'source {venv_name}/bin/activate &&\
              python3 {folder}/scripts/installers.py {project_name}'
    subprocess.run(command, shell=True)



# Module imports
from .normalizer import project_to_venv
from .initializer import run


def main(args):
    argv_length = len(args)
    if argv_length == 2:
        project_name = args[1]
        venv_name = project_to_venv(project_name)
    elif argv_length > 2:
        project_name = args[1]
        venv_name = args[2]
    else:
        project_name = 'mysite'
        venv_name = 'MySite'
    run(project_name, venv_name)



from setuptools import setup, find_packages


with open('readme.md', 'r') as fh:
	long_description = fh.read()
	fh.close()

setup(
    name='django-kick-off',
    version='0.0.2',
	url="https://github.com/ignacio-nava/django-kick-off",
	author="Ignacio Nava",
	author_email="nava_ignacio@outlook.com",
	description='Start a virtual environment and a fully configurated Django project with an app',
	py_modules='kick_off',
	package_dir={'': 'src'},
	packages=find_packages(where="src"),
	python_requires=">=3.6",
	classifiers=[
		'Environment :: Console',
		'Programming Language :: Python :: 3.6',
		'Programming Language :: Python :: 3.7',
		'Programming Language :: Python :: 3.8',
		'Programming Language :: Python :: 3.9',
		'License :: OSI Approved :: MIT License',
		'Operating System :: MacOS',
		'Topic :: Software Development',
	],
	long_description=long_description,
	long_description_content_type="text/markdown",
	extras_require={
		"dev": [
			"pytest>=3.6",
			"check-manifest",
		],
	},
)



"""mysite URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
import os

from django.contrib import admin
from django.urls import path, include
from django.views.static import serve

urlpatterns = [
    path('admin/', admin.site.urls),
    path('', include('home.urls')),
]

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Favicon
favicon_path = {
    'path': 'favicon.ico',
    'document_root': os.path.join(BASE_DIR, 'home/static'),
}
urlpatterns += [
    path('favicon.ico', serve, favicon_path)
]
